@@ -44,7 +44,11 @@ def build_inno(pyoxidizer_target=None, python=None, iscc=None, version=None):
         )
     else:
         inno.build_with_py2exe(
-            SOURCE_DIR, build_dir, pathlib.Path(python), iscc, version=version,
+            SOURCE_DIR,
+            build_dir,
+            pathlib.Path(python),
+            iscc,
+            version=version,
         )
 
 
@@ -60,6 +64,7 @@ def build_wix(
     extra_packages_script=None,
     extra_wxs=None,
     extra_features=None,
+    extra_pyoxidizer_vars=None,
 ):
     if not pyoxidizer_target and not python:
         raise Exception("--python required unless building with PyOxidizer")
@@ -101,7 +106,7 @@ def build_wix(
             "timestamp_url": sign_timestamp_url,
         }
 
-    fn(**kwargs)
+    fn(**kwargs, extra_pyoxidizer_vars=extra_pyoxidizer_vars)
 
 
 def get_parser():
@@ -164,6 +169,12 @@ def get_parser():
             "in the installer from the extra wxs files"
         ),
     )
+
+    sp.add_argument(
+        "--extra-pyoxidizer-vars",
+        help="json map of extra variables to pass to pyoxidizer",
+    )
+
     sp.set_defaults(func=build_wix)
 
     return parser
